@@ -276,9 +276,9 @@ class ActionGroup(Gtk.ActionGroup):
 
         if first_action is not None and on_change is not None:
             if user_data is None:
-                action.connect('changed', on_change)
+                first_action.connect('changed', on_change)
             else:
-                action.connect('changed', on_change, user_data)
+                first_action.connect('changed', on_change, user_data)
 
 ActionGroup = override(ActionGroup)
 __all__.append('ActionGroup')
@@ -411,10 +411,8 @@ class Dialog(Gtk.Dialog, Container):
             self.set_destroy_with_parent(True)
 
         # NO_SEPARATOR has been removed from Gtk 3
-        try:
-            if flags & Gtk.DialogFlags.NO_SEPARATOR:
-                self.set_has_separator(False)
-        except AttributeError:
+        if hasattr(Gtk.DialogFlags, "NO_SEPARATOR") and (flags & Gtk.DialogFlags.NO_SEPARATOR):
+            self.set_has_separator(False)
             import warnings
             warnings.warn("Gtk.DialogFlags.NO_SEPARATOR has been depricated since Gtk+-3.0", DeprecationWarning)
 
@@ -797,13 +795,6 @@ class TreeModel(Gtk.TreeModel):
 
             self.set_value(treeiter, i, value)
 
-    def _decode_value(self, treeiter, column, value):
-        type_ = self.get_column_type(column)
-        if type_ == GObject.TYPE_STRING and sys.version_info < (3, 0):
-            value = value.decode('UTF-8')
-
-        return value
-
     def _convert_value(self, treeiter, column, value):
             if value is None:
                 return
@@ -972,10 +963,6 @@ class ListStore(Gtk.ListStore, TreeModel, TreeSortable):
         value = self._convert_value(treeiter, column, value)
         Gtk.ListStore.set_value(self, treeiter, column, value)
 
-    def get_value(self, treeiter, column):
-        value = Gtk.ListStore.get_value(self, treeiter, column)
-        return self._decode_value(treeiter, column, value)
-
     def set(self, treeiter, *args):
 
         def _set_lists(columns, values):
@@ -1191,10 +1178,6 @@ class TreeStore(Gtk.TreeStore, TreeModel, TreeSortable):
         value = self._convert_value(treeiter, column, value)
         Gtk.TreeStore.set_value(self, treeiter, column, value)
 
-    def get_value(self, treeiter, column):
-        value = Gtk.TreeStore.get_value(self, treeiter, column)
-        return self._decode_value(treeiter, column, value)
-
     def set(self, treeiter, *args):
 
         def _set_lists(columns, values):
